@@ -62,6 +62,10 @@ MeshGenerator make_meshgenerator(const Grid& grid, const AtlasTool::Args& args)
         config.set("type", args.getString("generator"));
     }
 
+    if (args.has("3d")) {
+        config.set("3d", true);
+    }
+
     if (mpi::comm().size() > 1 || args.getBool("edges", false)) {
         config.set("3d", false);
     }
@@ -84,6 +88,9 @@ MeshGenerator make_meshgenerator(const Grid& grid, const AtlasTool::Args& args)
         config.set("halo", args.getInt("halo"));
     }
 
+    if (args.has("angle")) {
+        config.set("angle", args.getDouble("angle"));
+    }
 
     return MeshGenerator{config};
 }
@@ -124,6 +131,7 @@ public:
 private:
     std::string key;
     long halo;
+    bool nodes;
     bool edges;
     bool cells;
     bool brick;
@@ -138,8 +146,10 @@ private:
 //-----------------------------------------------------------------------------
 
 Meshgen2Gmsh::Meshgen2Gmsh(int argc, char** argv): AtlasTool(argc, argv) {
-    add_option(new SimpleOption<bool>("lonlat", "Output mesh in lon,lat coordinates"));
-    add_option(new SimpleOption<bool>("ij", "Output mesh in i,j coordinates"));
+    add_option(new SimpleOption<std::string>("coordinates", "Output mesh in given coordinates"));
+    add_option(
+        new SimpleOption<bool>("lonlat", "Output mesh in lon,lat coordinates (shorthand for --coordinates=lonlat)"));
+    add_option(new SimpleOption<bool>("ij", "Output mesh in i,j coordinates (shorthand for --coordinates=ij)"));
     add_option(new SimpleOption<bool>("3d",
                                       "Output mesh as sphere, and generate "
                                       "mesh connecting East and West in "
@@ -165,7 +175,8 @@ Meshgen2Gmsh::Meshgen2Gmsh(int argc, char** argv): AtlasTool(argc, argv) {
 
     add_option(new Separator("Advanced"));
     add_option(new SimpleOption<long>("halo", "Halo size"));
-    add_option(new SimpleOption<bool>("edges", "Build edge datastructure"));
+    add_option(new SimpleOption<bool>("nodes", "Build nodes datastructure"));
+    add_option(new SimpleOption<bool>("edges", "Build edges datastructure"));
     add_option(new SimpleOption<bool>("cells", "Build cells datastructure"));
     add_option(new SimpleOption<bool>("brick", "Build brick dual mesh"));
     add_option(new SimpleOption<bool>("stats", "Write statistics file"));
@@ -203,6 +214,8 @@ int Meshgen2Gmsh::execute(const Args& args) {
     key = "";
     args.get("grid.name", key);
 
+    nodes = false;
+    args.get("nodes", nodes);
     edges = false;
     args.get("edges", edges);
     cells = false;
@@ -285,7 +298,7 @@ int Meshgen2Gmsh::execute(const Args& args) {
     }
 
 
-    if (grid.projection().units() == "degrees") {
+    if ((grid.projection().units() == "degrees" && halo > 0) || nodes) {
         functionspace::NodeColumns nodes_fs(mesh, option::halo(halo));
     }
     else {
@@ -315,6 +328,7 @@ int Meshgen2Gmsh::execute(const Args& args) {
     bool lonlat             = args.getBool("lonlat", false);
     bool ij                 = args.getBool("ij", false);
     std::string coordinates = dim_3d ? "xyz" : lonlat ? "lonlat" : ij ? "ij" : "xy";
+    args.get("coordinates", coordinates);
 
     if (args.getBool("gmsh", true)) {
         bool torus = false;
