@@ -13,7 +13,7 @@
 #include <vector>
 
 #include "atlas/array.h"
-#include "atlas/array/ArrayUtil.h"
+#include "atlas/array/ArrayDataStore.h"
 #include "atlas/array/ArrayView.h"
 #include "atlas/array/DataType.h"
 #include "atlas/array/MakeView.h"
@@ -67,7 +67,7 @@ public:
         static_assert(sizeof...(UInts) > 0, "1");
         auto gt_storage    = create_gt_storage<Value, typename default_layout_t<sizeof...(dims)>::type>(dims...);
         using data_store_t = typename std::remove_pointer<decltype(gt_storage)>::type;
-        array_.data_store_ = std::unique_ptr<ArrayDataStore>(new GridToolsDataStore<data_store_t>(gt_storage));
+        array_.data_store_ = std::make_unique<GridToolsDataStore<data_store_t>>(gt_storage);
         array_.spec_       = make_spec(gt_storage, dims...);
     }
 
@@ -77,7 +77,7 @@ public:
         auto gt_storage    = create_gt_storage<Value, typename default_layout_t<sizeof...(dims)>::type,
                                             ::gridtools::alignment<Alignment>>(dims...);
         using data_store_t = typename std::remove_pointer<decltype(gt_storage)>::type;
-        array_.data_store_ = std::unique_ptr<ArrayDataStore>(new GridToolsDataStore<data_store_t>(gt_storage));
+        array_.data_store_ = std::make_unique<GridToolsDataStore<data_store_t>>(gt_storage);
         array_.spec_       = make_spec(gt_storage, dims...);
     }
 
@@ -230,7 +230,7 @@ public:
     void construct_with_layout(UInts... dims) {
         auto gt_data_store_ptr = create_gt_storage<Value, Layout>(dims...);
         using data_store_t     = typename std::remove_pointer<decltype(gt_data_store_ptr)>::type;
-        array_.data_store_ = std::unique_ptr<ArrayDataStore>(new GridToolsDataStore<data_store_t>(gt_data_store_ptr));
+        array_.data_store_ = std::make_unique<GridToolsDataStore<data_store_t>>(gt_data_store_ptr);
         array_.spec_       = make_spec(gt_data_store_ptr, dims...);
     }
 
@@ -249,7 +249,7 @@ public:
 
         Array* resized = Array::create<Value>(ArrayShape{(idx_t)c...});
 
-        array_initializer<sizeof...(c)>::apply(array_, *resized);
+        array_initializer::apply(array_, *resized);
         array_.replace(*resized);
         delete resized;
     }
@@ -258,7 +258,6 @@ public:
     void apply_resize(const ArrayShape& shape, std::integer_sequence<UInt, Indices...>) {
         return resize_variadic(shape[Indices]...);
     }
-
 private:
     ArrayT<Value>& array_;
 };
@@ -493,6 +492,11 @@ void ArrayT<Value>::resize(idx_t dim0, idx_t dim1, idx_t dim2, idx_t dim3, idx_t
     ArrayT_impl<Value>(*this).resize_variadic(dim0, dim1, dim2, dim3, dim4);
 }
 
+template <typename Value>
+void ArrayT<Value>::copy(const Array& other, const Array::CopyPolicy&) {
+        array_initializer::apply(other, *this);
+}
+
 template <typename Value>
 void ArrayT<Value>::dump(std::ostream& out) const {
     switch (rank()) {
