@@ -96,7 +96,8 @@ private:
 
     static value_type* create(const Mesh& mesh) {
         value_type* value = new value_type();
-        value->setup(array::make_view<int, 1>(mesh.edges().partition()).data(),
+        value->setup(mesh.mpi_comm(),
+                     array::make_view<int, 1>(mesh.edges().partition()).data(),
                      array::make_view<idx_t, 1>(mesh.edges().remote_index()).data(), REMOTE_IDX_BASE,
                      mesh.edges().size());
         return value;
@@ -130,7 +131,8 @@ private:
 
     static value_type* create(const Mesh& mesh) {
         value_type* value = new value_type();
-        value->setup(array::make_view<int, 1>(mesh.edges().partition()).data(),
+        value->setup(mesh.mpi_comm(),
+                     array::make_view<int, 1>(mesh.edges().partition()).data(),
                      array::make_view<idx_t, 1>(mesh.edges().remote_index()).data(), REMOTE_IDX_BASE,
                      array::make_view<gidx_t, 1>(mesh.edges().global_index()).data(), mesh.edges().size());
         return value;
@@ -194,7 +196,6 @@ void EdgeColumns::set_field_metadata(const eckit::Configuration& config, Field&
 }
 
 idx_t EdgeColumns::config_size(const eckit::Configuration& config) const {
-    const idx_t rank = mpi::rank();
     idx_t size       = nb_edges();
     bool global(false);
     if (config.get("global", global)) {
@@ -202,6 +203,7 @@ idx_t EdgeColumns::config_size(const eckit::Configuration& config) const {
             idx_t owner(0);
             config.get("owner", owner);
             idx_t _nb_edges_global(nb_edges_global());
+            const idx_t rank = mpi::comm(mpi_comm()).rank();
             size = (rank == owner ? _nb_edges_global : 0);
         }
     }
@@ -266,7 +268,7 @@ EdgeColumns::EdgeColumns(const Mesh& mesh, const eckit::Configuration& config):
         return _nb_edges;
     };
 
-    mesh::actions::build_nodes_parallel_fields(mesh_.nodes());
+    mesh::actions::build_nodes_parallel_fields(mesh_);
     mesh::actions::build_periodic_boundaries(mesh_);
 
     if (halo_.size() > 0) {
@@ -570,6 +572,10 @@ Field EdgeColumns::remote_index() const {
     return edges().remote_index();
 };
 
+Field EdgeColumns::partition() const {
+    return edges().partition();
+}
+
 //------------------------------------------------------------------------------
 //------------------------------------------------------------------------------
 //------------------------------------------------------------------------------
