@@ -441,6 +441,7 @@ void eq_regions(int N, double xmin[], double xmax[], double ymin[], double ymax[
 }
 
 void EqualRegionsPartitioner::init() {
+    N_ = nb_partitions();
     std::vector<double> s_cap;
     eq_caps(N_, sectors_, s_cap);
     bands_.resize(s_cap.size());
@@ -449,22 +450,35 @@ void EqualRegionsPartitioner::init() {
     }
 }
 
-EqualRegionsPartitioner::EqualRegionsPartitioner(): Partitioner(), N_(nb_partitions()) {
+EqualRegionsPartitioner::EqualRegionsPartitioner(): 
+    Partitioner() {
     init();
 }
 
-EqualRegionsPartitioner::EqualRegionsPartitioner(int N): Partitioner(N), N_(N) {
-    init();
+EqualRegionsPartitioner::EqualRegionsPartitioner(int N): 
+    EqualRegionsPartitioner(N, util::NoConfig()) {
 }
 
 EqualRegionsPartitioner::EqualRegionsPartitioner(int N, const eckit::Parametrisation& config):
-    EqualRegionsPartitioner(N) {
+    Partitioner(N,config) {
     std::string crds;
     if (config.get("coordinates", crds)) {
         if (crds == "lonlat") {
             coordinates_ = Coordinates::LONLAT;
         }
     }
+    init();
+}
+
+EqualRegionsPartitioner::EqualRegionsPartitioner(const eckit::Parametrisation& config):
+    Partitioner(config) {
+    std::string crds;
+    if (config.get("coordinates", crds)) {
+        if (crds == "lonlat") {
+            coordinates_ = Coordinates::LONLAT;
+        }
+    }
+    init();
 }
 
 int EqualRegionsPartitioner::partition(const double& x, const double& y) const {
@@ -481,14 +495,16 @@ int EqualRegionsPartitioner::band(const double& y) const {
 }
 
 int EqualRegionsPartitioner::sector(int band, const double& x) const {
+    constexpr double two_pi = 2. * M_PI;
+    constexpr double div_two_pi = 1. / (2. * M_PI + 1.e-8);
     double xreg = x;
     if (x < 0.) {
-        xreg += 2. * M_PI;
+        xreg += two_pi;
     }
-    else if (x > 2. * M_PI) {
-        xreg -= 2. * M_PI;
+    else if (x > two_pi) {
+        xreg -= two_pi;
     }
-    return std::floor(xreg * sectors_[band] / (2. * M_PI + 1e-8));
+    return std::floor(xreg * sectors_[band] * div_two_pi);
 }
 
 void EqualRegionsPartitioner::where(int partition, int& band, int& sector) const {
@@ -605,7 +621,7 @@ void EqualRegionsPartitioner::partition(const Grid& grid, int part[]) const {
 
         ATLAS_ASSERT(grid.projection().units() == "degrees");
 
-        const auto& comm = mpi::comm();
+        const auto& comm = mpi::comm(mpi_comm());
         int mpi_rank     = static_cast<int>(comm.rank());
         int mpi_size     = static_cast<int>(comm.size());
 
