@@ -82,7 +82,7 @@ struct Node {
         auto this_stack_hash   = TimingsRegistry::instance().stack_[index].hash();
         auto is_child          = [&](size_t i) -> bool {
             CallStack child_stack = TimingsRegistry::instance().stack_[i];
-            child_stack.pop_front();
+            child_stack.pop();
             auto child_stack_hash = child_stack.hash();
             return child_stack_hash == this_stack_hash;
         };
@@ -232,7 +232,7 @@ void TimingsRegistry::report(std::ostream& out, const eckit::Configuration& conf
     auto digits = [](long x) -> long { return std::floor(std::log10(std::max(1l, x))) + 1l; };
 
     std::vector<size_t> excluded_timers_vector;
-    for (auto label : labels_) {
+    for (auto& label : labels_) {
         auto name = label.first;
         if (excluded_labels.count(name)) {
             auto timers = label.second;
@@ -353,10 +353,9 @@ void TimingsRegistry::report(std::ostream& out, const eckit::Configuration& conf
             }
             const CallStack& next_stack = *next_stack_ptr;
 
-            auto this_it = this_stack.rbegin();
-            auto next_it = next_stack.rbegin();
-            for (size_t i = 0; this_it != this_stack.rend() && next_it != next_stack.rend();
-                 ++i, ++this_it, ++next_it) {
+            auto this_it = this_stack.begin();
+            auto next_it = next_stack.begin();
+            for (size_t i = 0; this_it != this_stack.end() && next_it != next_stack.end(); ++i, ++this_it, ++next_it) {
                 if (*this_it == *next_it) {
                     active[i] = active[i] or false;
                 }
@@ -428,7 +427,7 @@ void TimingsRegistry::report(std::ostream& out, const eckit::Configuration& conf
     out << std::left << std::setw(40) << "Timers accumulated by label" << sep << std::left << std::setw(5) << "count"
         << sep << "time" << std::endl;
     out << std::left << box_horizontal(40) << seph << box_horizontal(5) << seph << box_horizontal(12) << "\n";
-    for (auto label : labels_) {
+    for (auto& label : labels_) {
         auto name   = label.first;
         auto timers = label.second;
         double tot(0);
@@ -444,18 +443,15 @@ void TimingsRegistry::report(std::ostream& out, const eckit::Configuration& conf
 }
 
 std::string TimingsRegistry::filter_filepath(const std::string& filepath) const {
-    std::regex filepath_re("(.*)?/atlas/src/(.*)");
     std::smatch matches;
-    std::string filtered("");
-    if (std::regex_search(filepath, matches, filepath_re)) {
-        // filtered = matches[2];
-        filtered = "[atlas] ";
+    std::string basename = eckit::PathName(filepath).baseName();
+    if (std::regex_search(filepath, matches, std::regex{"(.*)?/atlas/src/(.*)"})) {
+        return "[atlas] " + basename;
     }
-    filtered += eckit::PathName(filepath).baseName();
-    return filtered;
-    //
-    // return filtered;
-    // return filepath;
+    if (std::regex_search(filepath, matches, std::regex{"(.*)?/atlas-io/src/(.*)"})) {
+        return "[atlas-io] " + basename;
+    }
+    return basename;
 }
 
 Timings::Identifier Timings::add(const CodeLocation& loc, const CallStack& stack, const std::string& title,
