@@ -11,11 +11,28 @@
 #include "eckit/geometry/Point2.h"
 #include "eckit/geometry/Point3.h"
 
+#include "atlas/library/config.h"
 #include "atlas/runtime/Exception.h"
 #include "atlas/util/Geometry.h"
 
 namespace atlas {
 
+namespace geometry {
+namespace detail {
+void GeometrySphere::lonlat2xyz(const Point2& lonlat, Point3& xyz) const {
+#if ATLAS_ECKIT_VERSION_AT_LEAST(1, 24, 0)
+    Sphere::convertSphericalToCartesian(radius_, lonlat, xyz, 0., true);
+#else
+    Sphere::convertSphericalToCartesian(radius_, lonlat, xyz);
+#endif
+}
+void GeometrySphere::xyz2lonlat(const Point3& xyz, Point2& lonlat) const {
+    Sphere::convertCartesianToSpherical(radius_, xyz, lonlat);
+}
+
+} // namespace detail
+} // namespace geometry
+
 extern "C" {
 // ------------------------------------------------------------------
 // C wrapper interfaces to C++ routines
